#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <sys/time.h>

#include "delaygrab.h"

#define BLOCKSIZE 1
#define QUEUEDEPTH 76

DelaygrabFilter::DelaygrabFilter() {
  delaymap=NULL;
  imagequeue=NULL;
  procbuf=NULL;
}

DelaygrabFilter::~DelaygrabFilter() {
  cerr << "-- DelaygrabFilter::~DelaygrabFilter\n";
  if(delaymap!=NULL) {
    fprintf(stderr,"   freeing delaymap[0x%x]\n",delaymap);
    free(delaymap);
  }
  if(imagequeue!=NULL) {
    fprintf(stderr,"   freeing delaymap[0x%x]\n",imagequeue);
    free(imagequeue);
  }
  if(procbuf!=NULL) {
    fprintf(stderr,"   freeing delaymap[0x%x]\n",procbuf);
    free(procbuf);
  }
}

bool DelaygrabFilter::init() {
  cerr << "-- DelaygrabFilter::init\n";

  delaymapwidth = w/BLOCKSIZE;
  delaymapheight = h/BLOCKSIZE;
  delaymapsize = delaymapheight*delaymapwidth;
  fprintf(stderr,"   delaymapwidth[%u] delaymapheight[%u] delaymapsize[%u]\n",delaymapwidth,delaymapheight,delaymapsize);
  cerr << "   seeding randomizer\n";
  srand(time(NULL));

  imagequeue=(unsigned char *)malloc(QUEUEDEPTH*size);
  if(imagequeue==NULL) {
    fprintf(stderr," ! Not enough memory for a %i frame queue (%i bytes)\n",QUEUEDEPTH,QUEUEDEPTH*size);
    return(false);
  } else
    fprintf(stderr,"   allocated imagequeue[0x%x] sized[%u]\n",imagequeue,QUEUEDEPTH*size);

  procbuf=(unsigned char *)malloc(size);
  if(procbuf==NULL) {
    fprintf(stderr," ! out of memory allocating procbuf sized[%u]\n",size);
    return(false);
  } else
    fprintf(stderr,"   allocated procbuf[0x%x] sized[%u]\n",procbuf,size);
  
  curqueue=imagequeue;
  curqueuenum=0;
  linewidth = w*(bpp>>3);

  createDelaymap(2);
    
  return(true);
}

void DelaygrabFilter::createDelaymap(int mode) {
  cerr << "-- DelaygrabFilter::createDelaymap\n";
  
  if(delaymap==NULL) {
    delaymap=(int *)malloc(delaymapsize*sizeof(int));
    fprintf(stderr,"   allocated delaymap[0x%x] sized[%u] for mode[%u]\n",delaymap,delaymapsize*sizeof(int),mode);
  }

  curdelaymap=delaymap;
  for (y=0; y<delaymapheight; y++) {
  for (x=0; x<delaymapwidth; x++) {
   switch (mode) {
   case 1:
     double d;
     /* Random delay with square distribution */
     d = (double)rand()/(double)RAND_MAX;
     *curdelaymap = (int)(d*d*16.0);
     break;
    case 2:
     /* Stripes of increasing delay outward from center */
     if (x<(delaymapwidth/2)) {
      v=(delaymapwidth/2)-x;
     } else if (x>(delaymapwidth/2)) {
      v=x-(delaymapwidth/2);
     } else {
      v=0;
     }
     *curdelaymap=v/2;
     break;
    case 3:
     /* Rings of increasing delay outward from center */
     v = (int)sqrt((double)((x-(delaymapwidth/2))*(x-(delaymapwidth/2))+(y-(delaymapheight/2))*(y-(delaymapheight/2))));
     *curdelaymap=v/2;
     break;
   } // switch
   /* Clip values */
   if (*curdelaymap<0) {
    *curdelaymap=0;
   } else if (*curdelaymap>(QUEUEDEPTH-1)) {
    *curdelaymap=(QUEUEDEPTH-1);
   }
   curdelaymap++;
  }
 }
}

unsigned char *DelaygrabFilter::process(unsigned char *buffo) {

  /* Update queue pointer */
  if (curqueuenum==0) {
    curqueuenum=QUEUEDEPTH-1;
    curqueue = imagequeue;
    curqueue += (size*(QUEUEDEPTH-1));
  } else {
    curqueuenum--;
    curqueue -= (size);
  }

   /* Copy image to queue */
  memcpy((unsigned char *)curqueue,(unsigned char *)buffo,size);

     /* Copy image blockwise to screenbuffer */
  curdelaymap=delaymap;
  for (y=0; y<delaymapheight; y++) {
    for (x=0; x<delaymapwidth; x++) {
     curposnum=((curqueuenum + (*curdelaymap)) % QUEUEDEPTH);
     xyoff= x*(BLOCKSIZE*(bpp>>3)) + y*(BLOCKSIZE*linewidth);
     // source
     curpos= (unsigned char *)imagequeue;
     curpos += (size*curposnum);
     curpos += xyoff;
     // target
     curimage = (unsigned char *)procbuf;
     curimage += xyoff;
     // copy block
     for (i=0; i<BLOCKSIZE; i++) {
       //       cerr << i;
       memcpy((unsigned char *)curimage,(unsigned char *)curpos,(BLOCKSIZE<<1));
       curpos += (linewidth);
       curimage += (linewidth);
     }
     curdelaymap++;
    }
   }

  return(procbuf);
}
