/* comet.cpp - part of comet
 */

#include <iostream>
#include <assert.h>
#include <context.h>
#include <v4l.h>
#include "filters/filter-list.h"

#define A 320
#define L 240
#define P 16

Context *screen = NULL;
V4lGrabber *grabber = NULL;
StrobeFilter *strobe = NULL;
TestFilter *test = NULL;
DelaygrabFilter *dgrab = NULL;
AbsdiffFilter *absdifffilt = NULL;

int main (int argc, char **argv) {
  
  bool quit = false;
  
  SDL_Event event;

  /* allocates SDL context */
  screen = new Context(A,L,P,SDL_HWSURFACE|SDL_DOUBLEBUF|SDL_FULLSCREEN);

  /* allocates v4l grabber */
  grabber = new V4lGrabber();
  if(!grabber->detect("/dev/video")) {
    delete grabber;
    delete screen;
    exit(0);
  }

  if(!grabber->init(screen)) {
    delete grabber;
    delete screen;
    exit(0);
  }
  /* --------------------- */
  
  /* allocates Test filter */
  //  strobe = new StrobeFilter();
  //  test = new TestFilter();
  dgrab = new DelaygrabFilter();
  //  absdifffilt =  new AbsdiffFilter();

  /* registers the filter */
  assert( grabber->add_filter(dgrab) );

  /* update the fps counter every 25 frames */
  screen->set_fps_interval(25);
  
  while(!quit) {
    while(SDL_PollEvent(&event)) {
      
      switch(event.type) {
      case SDL_QUIT:
	quit = true;
	break;
      case SDL_KEYDOWN:

	switch(event.key.keysym.sym) {
	  case SDLK_ESCAPE:
	  quit = true;
	  break;

	case SDLK_1:
	  dgrab->createDelaymap(1);
	  break;

	case SDLK_2:
	  dgrab->createDelaymap(2);
	  break;
	
	case SDLK_3:
	  dgrab->createDelaymap(3);
	  break;
	case SDLK_0:
	  SDL_WM_ToggleFullScreen(screen->surf);
	  break;
	  
	  
	}
      }
    }

    /* blit the layer on screen */
    assert( grabber->blit(screen->get_surface()) );
    
    /* flip the screen */
    assert( screen->flip() );

    /* feed the layer */
    assert( grabber->feed() );
    
    
    /* keep track of fps */
    //    if(!screen->calc_fps())
      /* and show them if counter says so (false)*/
    //      screen->print_fps();
  }

  if(absdifffilt!=NULL)
    delete absdifffilt;
  if(dgrab!=NULL)
    delete dgrab;
  if(test!=NULL)
    delete test;
  if(strobe!=NULL)
    delete strobe;
  if(screen!=NULL)
    delete screen;
  exit(0);

}
